﻿using notimeIn;
using System;
using System.Windows.Forms;
using YKCat2;
#if !DEBUG
using System.Text;
#endif

namespace notimeWin
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// C#应用名称
        /// </summary>
        readonly string APP_NAME = "notimeIn.exe";
        /// <summary>
        /// 正在监视
        /// </summary>
        bool mMonitoring = false;
        /// <summary>
        /// 库是否已经加载
        /// </summary>
        bool mLibHasInited = false;
        /// <summary>
        /// 刷新频率
        /// </summary>
        const int REFRESH_INTERVAL = 100;
        /// <summary>
        /// 采集定时器
        /// </summary>
        System.Timers.Timer mRefreshTimer;
        /// <summary>
        /// 退出标记
        /// </summary>
        bool mExitFlag = false;

        public MainForm()
        {
            InitializeComponent();
            Load += MainForm_Load;
            FormClosing += MainForm_FormClosing;

            string startPath = GetAssemblyPath();
            APP_NAME = $"{startPath}{APP_NAME}";
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                mExitFlag = true;
                System.Diagnostics.Debug.WriteLine(" mExitFlag = true");
                if (mMonitoring)
                {
                    Worker.StopTask();
                }
                mMonitoring = false;
                mLibHasInited = false;
            }
            catch //(Exception ex)
            {

            }
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            InitTimer();
        }


        /// <summary>
        /// 获取Assembly的运行路径
        /// </summary>
        ///<returns></returns>
        private static string GetAssemblyPath()
        {
            string _CodeBase = System.Reflection.Assembly.GetExecutingAssembly().CodeBase;
            _CodeBase = _CodeBase.Substring(8, _CodeBase.Length - 8);    // 8是file:// 的长度
            int index = _CodeBase.LastIndexOf('/');
            _CodeBase = _CodeBase.Substring(0, index + 1);
            if (!string.IsNullOrEmpty(_CodeBase))
            {
                _CodeBase = _CodeBase.Replace('/', '\\');
            }
            return _CodeBase;
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            try
            {
                if (mMonitoring)
                {
                    return;
                }
                RetCode ret = RetCode.RET_OK;
                if (!mLibHasInited)
                {
                    ret = ProCon.YKM_LoadLib();
                    if (ret != RetCode.RET_OK)
                    {
                        ShowMessage($"YKM_LoadLib 失败({ret})");
                        return;
                    }
                }
#if !DEBUG
                byte[] name = Encoding.UTF8.GetBytes(APP_NAME);
                ret = ProCon.YKM_LoadNotimeApp(name);
                if (ret != RetCode.RET_OK)
                {
                    ShowMessage($"YKM_LoadNotimeApp 失败({ret})");
                    return;
                }
                System.Threading.Thread.Sleep(1000);
#endif
               var  ret1 = notimeIn.MemOper.OpenShareMemory();
                if (ret1 != NoLib.NoCode.RET_OK)
                {
                    ShowMessage($"OpenShareMemory 失败({ret1})");
                    return;
                }
                Worker.StartTask();
                mLibHasInited = true;
                mRefreshTimer.Start();
                mMonitoring = true;
                btnStart.Enabled = false;
            }
            catch (Exception ex)
            {
                ShowMessage($"打开失败({ex.Message})");
            }
        }

        /// <summary>
        /// 初始化采集定时器
        /// </summary>
        private void InitTimer()
        {
            mRefreshTimer = new System.Timers.Timer(REFRESH_INTERVAL);
            mRefreshTimer.AutoReset = false;
            mRefreshTimer.Elapsed += MRefreshTimer_Elapsed;
        }

        private void MRefreshTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            try
            {
                var ticks = MemOper.GetTimes();
                txtTime.Invoke(new Action(() =>
                {
                    this.txtTime.Text = ticks.ToString();
                }));

            }
            catch //(Exception ex)
            {

            }
            finally
            {
                if (!mExitFlag)
                {
                    mRefreshTimer.Start();
                }
            }
        }

        public static void ShowMessage(string msg)
        {
            MessageBox.Show(msg, "系统提示", MessageBoxButtons.OK, MessageBoxIcon.Error);

        }
    }
}
