﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using NoLib;

namespace NotimeIntegrationEG_Notime
{
    public class MemOper
    {
        /// <summary>
        ///  共享内存名称
        /// </summary>
        const string MEM_NAME = "memtick";

        /// <summary>
        /// 共享内存指针
        /// </summary>
        public static ulong memAddr = 0;

        /// <summary>
        /// 结构体指针
        /// </summary>
        unsafe public static MemConfig* pConfig
        {
            get;
            private set;
        }
        /// <summary>
        /// 创建共享内存
        /// </summary>
        /// <returns></returns>
        unsafe public static NoCode CreateShareMemory()
        {
            int size = Marshal.SizeOf(new MemConfig());
            ulong addr = 0;
            NoCode code = NoSys.NOS_CreateShareMemory(MEM_NAME, (uint)size, out addr);
            pConfig = (MemConfig*)addr;
            memAddr = addr;
            return code;
        }

        /// <summary>
        /// 打开共享内存
        /// </summary>
        /// <returns></returns>
        unsafe public static NoCode OpenShareMemory()
        {
            ulong addr = 0;
            NoCode code = NoSys.NOS_OpenShareMemory(MEM_NAME, out addr);
            pConfig = (MemConfig*)addr;
            return code;
        }

        /// <summary>
        /// 启动
        /// </summary>
        unsafe public static void Start()
        {
            if (null == pConfig)
                return;
            pConfig->Start = 1;
        }

        /// <summary>
        /// 停止
        /// </summary>
        unsafe public static void Stop()
        {
            if (null == pConfig)
                return;
            pConfig->Start = 0;
        }

        unsafe public static bool GetStart()
        {
            if (null == pConfig)
                return false;
            return pConfig->Start != 0;
        }

        /// <summary>
        /// 退出
        /// </summary>
        unsafe public static void Exit()
        {
            if (null == pConfig)
                return;
            pConfig->Exit = 1;
        }

        unsafe public static bool GetExit()
        {
            if (null == pConfig)
                return true;
            return pConfig->Exit != 0;
        }

        unsafe public static double GetTargetPosA()
        {
            if (null == pConfig)
                return 0;
            return pConfig->TargetPosA;
        }
        unsafe public static double GetTargetPosB()
        {
            if (null == pConfig)
                return 0;
            return pConfig->TargetPosB;
        }

        unsafe public static void SetTargetPosA(double targetPosA)
        {
            if (null == pConfig)
                return;
            pConfig->TargetPosA = targetPosA;
        }
        unsafe public static void SetTargetPosB(double targetPosB)
        {
            if (null == pConfig)
                return;
            pConfig->TargetPosB = targetPosB;
        }
    }

    /// <summary>
    /// 共享内存结构
    /// </summary>
    unsafe public struct MemConfig
    {
        /// <summary>
        /// 流程启动
        /// </summary>
        public int Start;

        /// <summary>
        /// 程序退出
        /// </summary>
        public int Exit;

        /// <summary>
        /// TargetPosA
        /// </summary>
        public double TargetPosA;

        /// <summary>
        /// TargetPosB
        /// </summary>
        public double TargetPosB;
    }
}
