﻿using NoLib;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Runtime.InteropServices;

namespace NoLib
{
    public enum NoCode
    {
        //正常
        RET_OK = 0,

        //不支持的函数
        RET_FUN = 1,

        //创建共享内存错误
        RET_CREATE_SH = 2,

        //打开共享内存错误
        RET_OPEN_SH = 3,

        //参数为空
        RET_NULL = 4,

        //参数越界
        RET_PARA = 5,

        //定时器错误
        RET_TIMER = 6,
    }

    public class NoSys
    {
        /// <summary>
        /// 获取版本号
        /// </summary>
        /// <param name="version">版本号返回值</param>
        /// <returns></returns>
        [DllImport("NoSys.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern NoCode NOS_GetVersion(out int version);

        /// <summary>
        /// 获取时钟(微秒)
        /// </summary>
        /// <param name="tick">时钟返回值</param>
        /// <returns></returns>
        [DllImport("NoSys.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern NoCode NOS_GetUsTick(out UInt64 tick);

        /// <summary>
        /// 创建共享内存
        /// </summary>
        /// <param name="name"></param>
        /// <param name="int_size"></param>
        /// <param name="handle"></param>
        /// <returns></returns>
        [DllImport("NoSys.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern NoCode NOS_CreateShareMemory(string name, UInt32 int_size, out ulong handle);

        /// <summary>
        /// 打开共享内存, 默认NodeA节点
        /// </summary>
        /// <param name="name"></param>
        /// <param name="handle"></param>
        /// <returns></returns>
        [DllImport("NoSys.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern NoCode NOS_OpenShareMemory(string name, out ulong handle);

        /// <summary>
        /// 打开共享内存
        /// </summary>
        /// <param name="nodeIndex">节点选择 0=NodeA 1=NodeB 2=NodeC 3=NodeD</param>
        /// <param name="memName">共享内存名称</param>
        /// <param name="oHandle">首地址</param>
        /// <returns></returns>
        [DllImport("NoSys.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern NoCode NOS_OpenShareMemoryEx(UInt32 nodeIndex, string memName, out ulong oHandle);

        /// <summary>
        /// 写共享内存
        /// </summary>
        /// <param name="handle">共享内存首地址</param>
        /// <param name="offset">偏移量</param>
        /// <param name="data">写入数据</param>
        /// <param name="num">数量，字节单位</param>
        /// <returns></returns>
        [DllImport("NoSys.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern NoCode NOS_WriteShareMemory(ulong handle, int offset, Byte[] data, int num);

        /// <summary>
        /// 读共享内存
        /// </summary>
        /// <param name="handle">共享内存首地址</param>
        /// <param name="offset">偏移量</param>
        /// <param name="data">返回数据</param>
        /// <param name="num">数量，字节单位</param>
        /// <returns></returns>
        [DllImport("NoSys.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern NoCode NOS_ReadShareMemory(ulong handle, int offset, Byte[] data, int num);

        /// <summary>
        /// 创建实时定时器
        /// </summary>
        /// <param name="interval">定时间隔，微秒单位，最小250us</param>
        /// <param name="callBack"></param>
        /// <param name="index">返回的定时器编号，最多支持16个定时器</param>
        /// <returns></returns>
        [DllImport("NoSys.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern NoCode NOS_CreateRtTimer(UInt32 interval, TimerCallbackFun callBack, out int index);

        /// <summary>
        /// 启动实时定时器
        /// </summary>
        /// <param name="index">定时器编号</param>
        /// <returns></returns>
        [DllImport("NoSys.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern NoCode NOS_StartRtTimer(int index);

        /// <summary>
        /// 停止实时定时器
        /// </summary>
        /// <param name="index">定时器编号</param>
        /// <returns></returns>
        [DllImport("NoSys.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern NoCode NOS_StopRtTimer(int index);

        /// <summary>
        /// 销毁实时定时器
        /// </summary>
        /// <param name="index">定时器编号</param>
        /// <returns></returns>
        [DllImport("NoSys.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern NoCode NOS_DisposeRtTimer(int index);

        public delegate void TimerCallbackFun(/*ulong ticks, object usrData*/);
    }
}


